﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Arction.WPF.SignalTools;


namespace WpfApplication1
{
    /// <summary>
    /// This app demonstrates how to feed double[][] data to AudioOuput component. 
    /// </summary>
    public partial class MainWindow : Window
    {

        
        AudioOutput _ao;
        MultiChannelDataPlaybackFeeder _feeder;

        public MainWindow()
        {
            InitializeComponent();

            this.Closing += new System.ComponentModel.CancelEventHandler(MainWindow_Closing);
            

            //Create stereo waveform arrays, both with same sine data. 
            int iSampleRateForDataFeed = 10000; 
            double dSineFreq = 400; 
            int iDurationSeconds = 10; 
            int iSampleCount = iSampleRateForDataFeed * iDurationSeconds; 
            int iChannelCount = 2; 
            double[][] samples = new double[iChannelCount][]; 
            double dAmplitude = 1000;
            
            for(int iCh = 0; iCh < iChannelCount; iCh++)
            {
                samples[iCh] = new double[iSampleCount]; 
                for(int i=0;i< iSampleCount;i++)
                {
                    //Create sine wave
                    samples[iCh][i] = dAmplitude *  Math.Sin(2.0 * Math.PI * dSineFreq * (double)i / (double)iSampleRateForDataFeed); 
                }
            }

            //Create Audio output 
            _ao = new AudioOutput();
            _ao.BitsPerSample = 16;
            _ao.SamplesPerSecond = iSampleRateForDataFeed;
            _ao.IsStereo = true; 
            _ao.Start();

            
            //Create multichannenl sample data feeder. Give sample arrays as parameter. 
            _feeder = new MultiChannelDataPlaybackFeeder(samples, iSampleRateForDataFeed, 1);
            _feeder.DataGenerated += new MultiChannelDataPlaybackFeeder.DataGeneratedEventHandler(feeder_DataGenerated);
            _feeder.Start(); 

        }


        void feeder_DataGenerated(double[][] newSamples)
        {
            //Note: DataGenerated is not thread-safe. It's OK to pass the data to AudioOutput
            //but ensure you are using Dispatcher.Invoke if you give the data for the LightningChart, otherwise it will crash. 
            _ao.AddSamples(newSamples);
        }

        void MainWindow_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            _feeder.Stop();
            _ao.StopRequest(); 
        }

      


    }
}
