﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading; 
using System.Threading.Tasks; 

namespace WpfApplication1
{
    public class MultiChannelDataPlaybackFeeder
    {
        
        private uint _uiPlaybackIntervalMs = 1;
        private double[][] _data;
        private long _lSampleBundleCount;
        private int _iChannelCount;
        private long _lTicksNow;
        private long _lTicksLastOutputted;
        private double _dSamplingFrequency;
        private long _lStartTicks;
        private long _lOutputIntervalTicks;
        private long _lSamplesOutputted;
        private double _dSampleInterval;
        private bool _isRunning = false;
        private CancellationTokenSource _cancel; 
        public delegate void DataGeneratedEventHandler(double[][] newSamples);
        public event DataGeneratedEventHandler DataGenerated;

        public MultiChannelDataPlaybackFeeder(double[][] data, double samplesPerSec, uint playbackIntervalMs)
        {
            _uiPlaybackIntervalMs = playbackIntervalMs;
            _data = data;
            _dSamplingFrequency = samplesPerSec; 
           
            _iChannelCount = data.GetLength(0);
            _lSampleBundleCount = data[0].GetLength(0);
        }

        private void ThreadProc()
        {
            while (!_cancel.IsCancellationRequested)
            {
                _lTicksNow = DateTime.Now.Ticks;
                if ((_lTicksNow - _lTicksLastOutputted) > _lOutputIntervalTicks)
                {
                    long lSampleNumberNow = (long)(TimeSpan.FromTicks(_lTicksNow - _lStartTicks).TotalSeconds * _dSamplingFrequency);
                    int iOutputSampleBundleCount = (int)(lSampleNumberNow - _lSamplesOutputted);

                    if (iOutputSampleBundleCount + _lSamplesOutputted > _lSampleBundleCount)
                    {
                        iOutputSampleBundleCount = (int)(_lSampleBundleCount - _lSamplesOutputted); //deliver the rest 
                    }
                    double[][] aSamples = new double[_iChannelCount][];

                    if (iOutputSampleBundleCount > 0)
                    {
                        for (int iCh = 0; iCh < _iChannelCount; iCh++)
                        {
                            aSamples[iCh] = new double[iOutputSampleBundleCount];
                            Array.Copy(_data[iCh], _lSamplesOutputted, aSamples[iCh], 0, iOutputSampleBundleCount);
                        }

                        if (DataGenerated != null)
                        {
                            DataGenerated(aSamples);
                            System.Diagnostics.Debug.WriteLine("perse"); 
                        }

                        _lSamplesOutputted += iOutputSampleBundleCount;
                    }

                    _lTicksLastOutputted = _lTicksNow;
                    Thread.Sleep((int) _uiPlaybackIntervalMs); 
                }
                
            }

            _isRunning = false; 
        }


       

        public void Start()
        {
            _lStartTicks = DateTime.Now.Ticks;
            _lOutputIntervalTicks = TimeSpan.FromMilliseconds(_uiPlaybackIntervalMs).Ticks;
            _lTicksLastOutputted = _lStartTicks;
            _dSampleInterval = 1.0 /_dSamplingFrequency;
            _lSamplesOutputted = 0;

            _cancel = new CancellationTokenSource();

            Task task = new Task(() => { ThreadProc(); }, _cancel.Token);
            task.Start();
            
            _isRunning  = true; 

        }


        /// <summary>
        /// Stop the feeder. 
        /// </summary>
        /// <remarks>Stop has to be called before closing or disposing the parent form. Otherwise timer leaks memory.</remarks>
        public void Stop()
        {
            _cancel.Cancel(); 
        }

    }
}
